@@ -1,15 +1,16 @@
 # -*- coding: utf-8 -*-
 """Setup tests"""
-# pylint:disable=too-many-lines
 import os
 
 import networkx as nx
+import numpy as np
 import pytest
 from pymatgen.analysis.graphs import MoleculeGraph, StructureGraph
 from pymatgen.analysis.local_env import JmolNN
 from pymatgen.core import Molecule, Structure
 
 from moffragmentor import MOF
+from moffragmentor.sbu import Linker, Node
 
 THIS_DIR = os.path.dirname(os.path.realpath(__file__))
 
@@ -398,44 +399,44 @@ Finished data file "/Users/kevinmaikjablonka/Downloads/test_systre.cgd".
 """
 
 
-@pytest.fixture(scope="module")
+@pytest.fixture(scope="session")
 def get_agn_mof():
     mof = MOF.from_cif(os.path.join(THIS_DIR, "test_files", "RSM3317.cif"))
     return mof
 
 
-@pytest.fixture(scope="module")
+@pytest.fixture(scope="session")
 def get_cuiiibtc_mof():
     mof = MOF.from_cif(os.path.join(THIS_DIR, "test_files", "KAJZIH_freeONLY.cif"))
     return mof
 
 
-@pytest.fixture(scope="module")
+@pytest.fixture(scope="session")
 def get_hkust_mof():
     mof = MOF.from_cif(os.path.join(THIS_DIR, "test_files", "HKUST-1.cif"))
     return mof
 
 
-@pytest.fixture(scope="module")
+@pytest.fixture(scope="session")
 def get_mof5():
     mof = MOF.from_cif(os.path.join(THIS_DIR, "test_files", "mof-5_cellopt.cif"))
     return mof
 
 
-@pytest.fixture(scope="module")
+@pytest.fixture(scope="session")
 def get_single_metal_mof():
     # PIQMUZ_clean
     mof = MOF.from_cif(os.path.join(THIS_DIR, "test_files", "RSM3434.cif"))
     return mof
 
 
-@pytest.fixture(scope="module")
+@pytest.fixture(scope="session")
 def get_porphryin_mof():
     mof = MOF.from_cif(os.path.join(THIS_DIR, "test_files", "porphyrin_mof.cif"))
     return mof
 
 
-@pytest.fixture(scope="module")
+@pytest.fixture(scope="session")
 def get_hypothetical_mof():
     mof = MOF.from_cif(
         os.path.join(THIS_DIR, "test_files", "bcs_v1-litcic_1B_4H_Ch_opt_charge.cif")
@@ -443,26 +444,26 @@ def get_hypothetical_mof():
     return mof
 
 
-@pytest.fixture(scope="module")
+@pytest.fixture(scope="session")
 def get_p_linker_with_floating():
     mof = MOF.from_cif(os.path.join(THIS_DIR, "test_files", "MAGBON.cif"))
     return mof
 
 
-@pytest.fixture(scope="module")
+@pytest.fixture(scope="session")
 def get_li_mof_with_floating():
     mof = MOF.from_cif(os.path.join(THIS_DIR, "test_files", "LISZOE.cif"))
     return mof
 
 
-@pytest.fixture(scope="module")
+@pytest.fixture(scope="session")
 def abaxin():
     # https://github.com/kjappelbaum/moffragmentor/issues/70
     mof = MOF.from_cif(os.path.join(THIS_DIR, "test_files", "ABAXIN.cif"))
     return mof
 
 
-@pytest.fixture(scope="module")
+@pytest.fixture(scope="session")
 def get_1d_node_with_floating():
     """https://pubs.rsc.org/en/content/articlelanding/2014/cc/c3cc49684h#!divAbstract
 
@@ -475,7 +476,14 @@ def get_1d_node_with_floating():
     return mof
 
 
-@pytest.fixture(scope="module")
+@pytest.fixture(scope="session")
+def get_single_metal_mof_previously_failing():
+    """Issue #119"""
+    mof = MOF.from_cif(os.path.join(THIS_DIR, "test_files", "BAMHIJ.cif"))
+    return mof
+
+
+@pytest.fixture(scope="session")
 def get_1d_node_graph():
     not_node = [
         1,
@@ -624,7 +632,7 @@ def get_1d_node_graph():
     return graph
 
 
-@pytest.fixture(scope="module")
+@pytest.fixture(scope="session")
 def get_0d_node_graph():
     not_node = [
         1,
@@ -1245,29 +1253,29 @@ def get_0d_node_graph():
     return graph
 
 
-@pytest.fixture(scope="module")
+@pytest.fixture(scope="session")
 def get_cgd_file():
     return HKUST_cdg
 
 
-@pytest.fixture(scope="module")
+@pytest.fixture(scope="session")
 def get_systre_output():
     return SYSTRE_OUT
 
 
-@pytest.fixture(scope="module")
+@pytest.fixture(scope="session")
 def get_systre_output2():
     return SYSTRE_OUT_2
 
 
-@pytest.fixture(scope="module")
+@pytest.fixture(scope="session")
 def get_formate_structure_and_graph():
     s = Structure.from_file(os.path.join(THIS_DIR, "test_files", "WIHSUD_clean.cif"))
     sg = StructureGraph.with_local_env_strategy(s, JmolNN())
     return s, sg
 
 
-@pytest.fixture(scope="module")
+@pytest.fixture(scope="session")
 def get_dicarboxy_biphenyl_graph():
     graph = nx.read_graphml(
         os.path.join(THIS_DIR, "test_files", "test_graph_biphenyl_dicarboxy.graphml")
@@ -1326,3 +1334,380 @@ def get_across_periodic_boundary_node():
     # Zn4(μ3-OH)2(TPO)2(H2O)2]
     mof = MOF.from_cif(os.path.join(THIS_DIR, "test_files", "NIYZIG.cif"))
     return mof
+
+
+@pytest.fixture()
+def get_linker_object():
+    mg_dict = {
+        "@module": "pymatgen.analysis.graphs",
+        "@class": "MoleculeGraph",
+        "molecule": {
+            "@module": "pymatgen.core.structure",
+            "@class": "Molecule",
+            "charge": 0.0,
+            "spin_multiplicity": 2,
+            "sites": [
+                {
+                    "name": "H",
+                    "species": [{"element": "H", "occu": 1}],
+                    "xyz": [-42.67565999999999, -36.353339999999996, -45.520703999999995],
+                    "properties": {},
+                },
+                {
+                    "name": "O",
+                    "species": [{"element": "O", "occu": 1}],
+                    "xyz": [-45.915848999999994, -38.14466399999999, -44.335269],
+                    "properties": {},
+                },
+                {
+                    "name": "C",
+                    "species": [{"element": "C", "occu": 1}],
+                    "xyz": [-44.203554000000004, -36.537741000000004, -44.203554],
+                    "properties": {},
+                },
+                {
+                    "name": "O",
+                    "species": [{"element": "O", "occu": 1}],
+                    "xyz": [-40.884336, -33.11315100000001, -44.335269],
+                    "properties": {},
+                },
+                {
+                    "name": "C",
+                    "species": [{"element": "C", "occu": 1}],
+                    "xyz": [-43.044461999999996, -35.958195, -44.75675699999999],
+                    "properties": {},
+                },
+                {
+                    "name": "O",
+                    "species": [{"element": "O", "occu": 1}],
+                    "xyz": [-45.915849, -34.69373100000001, -40.884336],
+                    "properties": {},
+                },
+                {
+                    "name": "H",
+                    "species": [{"element": "H", "occu": 1}],
+                    "xyz": [-45.52070399999999, -36.35333999999999, -42.67565999999999],
+                    "properties": {},
+                },
+                {
+                    "name": "C",
+                    "species": [{"element": "C", "occu": 1}],
+                    "xyz": [-44.203554, -34.825446, -42.491259],
+                    "properties": {},
+                },
+                {
+                    "name": "C",
+                    "species": [{"element": "C", "occu": 1}],
+                    "xyz": [-44.862129, -34.166871, -41.332167],
+                    "properties": {},
+                },
+                {
+                    "name": "C",
+                    "species": [{"element": "C", "occu": 1}],
+                    "xyz": [-43.044461999999996, -34.272243, -43.044461999999996],
+                    "properties": {},
+                },
+                {
+                    "name": "O",
+                    "species": [{"element": "O", "occu": 1}],
+                    "xyz": [-44.335269, -33.113150999999995, -40.88433599999999],
+                    "properties": {},
+                },
+                {
+                    "name": "O",
+                    "species": [{"element": "O", "occu": 1}],
+                    "xyz": [-44.335269, -38.144664, -45.915848999999994],
+                    "properties": {},
+                },
+                {
+                    "name": "C",
+                    "species": [{"element": "C", "occu": 1}],
+                    "xyz": [-44.75675699999999, -35.958194999999996, -43.044461999999996],
+                    "properties": {},
+                },
+                {
+                    "name": "C",
+                    "species": [{"element": "C", "occu": 1}],
+                    "xyz": [-41.332167000000005, -34.166871, -44.862128999999996],
+                    "properties": {},
+                },
+                {
+                    "name": "C",
+                    "species": [{"element": "C", "occu": 1}],
+                    "xyz": [-42.49125899999999, -34.825446, -44.203554],
+                    "properties": {},
+                },
+                {
+                    "name": "H",
+                    "species": [{"element": "H", "occu": 1}],
+                    "xyz": [-42.67566, -33.508296, -42.67565999999999],
+                    "properties": {},
+                },
+                {
+                    "name": "O",
+                    "species": [{"element": "O", "occu": 1}],
+                    "xyz": [-40.884336, -34.693731, -45.915849],
+                    "properties": {},
+                },
+                {
+                    "name": "C",
+                    "species": [{"element": "C", "occu": 1}],
+                    "xyz": [-44.862129, -37.696833, -44.862128999999996],
+                    "properties": {},
+                },
+            ],
+        },
+        "graphs": {
+            "directed": True,
+            "multigraph": True,
+            "graph": [("edge_weight_name", None), ("edge_weight_units", None), ("name", "bonds")],
+            "nodes": [
+                {
+                    "idx": 96,
+                    "specie": "H",
+                    "coords": np.array([-42.67566, -36.35334, -45.520704]),
+                    "properties": {},
+                    "coord": np.array([-42.67566, -36.35334, -45.520704]),
+                    "id": 0,
+                },
+                {
+                    "idx": 129,
+                    "specie": "O",
+                    "coords": np.array([-45.915849, -38.144664, -44.335269]),
+                    "properties": {},
+                    "coord": np.array([-45.915849, -38.144664, -44.335269]),
+                    "id": 1,
+                },
+                {
+                    "idx": 23,
+                    "specie": "C",
+                    "coords": np.array([-44.203554, -36.537741, -44.203554]),
+                    "properties": {},
+                    "coord": np.array([-44.203554, -36.537741, -44.203554]),
+                    "id": 2,
+                },
+                {
+                    "idx": 152,
+                    "specie": "O",
+                    "coords": np.array([-40.884336, -33.113151, -44.335269]),
+                    "properties": {},
+                    "coord": np.array([-40.884336, -33.113151, -44.335269]),
+                    "id": 3,
+                },
+                {
+                    "idx": 60,
+                    "specie": "C",
+                    "coords": np.array([-43.044462, -35.958195, -44.756757]),
+                    "properties": {},
+                    "coord": np.array([-43.044462, -35.958195, -44.756757]),
+                    "id": 4,
+                },
+                {
+                    "idx": 137,
+                    "specie": "O",
+                    "coords": np.array([-45.915849, -34.693731, -40.884336]),
+                    "properties": {},
+                    "coord": np.array([-45.915849, -34.693731, -40.884336]),
+                    "id": 5,
+                },
+                {
+                    "idx": 105,
+                    "specie": "H",
+                    "coords": np.array([-45.520704, -36.35334, -42.67566]),
+                    "properties": {},
+                    "coord": np.array([-45.520704, -36.35334, -42.67566]),
+                    "id": 6,
+                },
+                {
+                    "idx": 0,
+                    "specie": "C",
+                    "coords": np.array([-44.203554, -34.825446, -42.491259]),
+                    "properties": {},
+                    "coord": np.array([-44.203554, -34.825446, -42.491259]),
+                    "id": 7,
+                },
+                {
+                    "idx": 1,
+                    "specie": "C",
+                    "coords": np.array([-44.862129, -34.166871, -41.332167]),
+                    "properties": {},
+                    "coord": np.array([-44.862129, -34.166871, -41.332167]),
+                    "id": 8,
+                },
+                {
+                    "idx": 2,
+                    "specie": "C",
+                    "coords": np.array([-43.044462, -34.272243, -43.044462]),
+                    "properties": {},
+                    "coord": np.array([-43.044462, -34.272243, -43.044462]),
+                    "id": 9,
+                },
+                {
+                    "idx": 108,
+                    "specie": "O",
+                    "coords": np.array([-44.335269, -33.113151, -40.884336]),
+                    "properties": {},
+                    "coord": np.array([-44.335269, -33.113151, -40.884336]),
+                    "id": 10,
+                },
+                {
+                    "idx": 141,
+                    "specie": "O",
+                    "coords": np.array([-44.335269, -38.144664, -45.915849]),
+                    "properties": {},
+                    "coord": np.array([-44.335269, -38.144664, -45.915849]),
+                    "id": 11,
+                },
+                {
+                    "idx": 69,
+                    "specie": "C",
+                    "coords": np.array([-44.756757, -35.958195, -43.044462]),
+                    "properties": {},
+                    "coord": np.array([-44.756757, -35.958195, -43.044462]),
+                    "id": 12,
+                },
+                {
+                    "idx": 37,
+                    "specie": "C",
+                    "coords": np.array([-41.332167, -34.166871, -44.862129]),
+                    "properties": {},
+                    "coord": np.array([-41.332167, -34.166871, -44.862129]),
+                    "id": 13,
+                },
+                {
+                    "idx": 14,
+                    "specie": "C",
+                    "coords": np.array([-42.491259, -34.825446, -44.203554]),
+                    "properties": {},
+                    "coord": np.array([-42.491259, -34.825446, -44.203554]),
+                    "id": 14,
+                },
+                {
+                    "idx": 84,
+                    "specie": "H",
+                    "coords": np.array([-42.67566, -33.508296, -42.67566]),
+                    "properties": {},
+                    "coord": np.array([-42.67566, -33.508296, -42.67566]),
+                    "id": 15,
+                },
+                {
+                    "idx": 120,
+                    "specie": "O",
+                    "coords": np.array([-40.884336, -34.693731, -45.915849]),
+                    "properties": {},
+                    "coord": np.array([-40.884336, -34.693731, -45.915849]),
+                    "id": 16,
+                },
+                {
+                    "idx": 46,
+                    "specie": "C",
+                    "coords": np.array([-44.862129, -37.696833, -44.862129]),
+                    "properties": {},
+                    "coord": np.array([-44.862129, -37.696833, -44.862129]),
+                    "id": 17,
+                },
+            ],
+            "adjacency": [
+                [{"to_jimage": (0, 0, 0), "id": 4, "key": 0}],
+                [{"to_jimage": (0, 0, 0), "id": 17, "key": 0}],
+                [
+                    {"to_jimage": (0, 0, 0), "id": 17, "key": 0},
+                    {"to_jimage": (0, 0, 0), "id": 12, "key": 0},
+                    {"to_jimage": (0, 0, 0), "id": 4, "key": 0},
+                ],
+                [{"to_jimage": (0, 0, 0), "id": 13, "key": 0}],
+                [
+                    {"to_jimage": (0, 0, 0), "id": 0, "key": 0},
+                    {"to_jimage": (0, 0, 0), "id": 2, "key": 0},
+                    {"to_jimage": (0, 0, 0), "id": 14, "key": 0},
+                ],
+                [{"to_jimage": (0, 0, 0), "id": 8, "key": 0}],
+                [{"to_jimage": (0, 0, 0), "id": 12, "key": 0}],
+                [
+                    {"to_jimage": (0, 0, 0), "id": 12, "key": 0},
+                    {"to_jimage": (0, 0, 0), "id": 8, "key": 0},
+                    {"to_jimage": (0, 0, 0), "id": 9, "key": 0},
+                ],
+                [
+                    {"to_jimage": (0, 0, 0), "id": 5, "key": 0},
+                    {"to_jimage": (0, 0, 0), "id": 7, "key": 0},
+                    {"to_jimage": (0, 0, 0), "id": 10, "key": 0},
+                ],
+                [
+                    {"to_jimage": (0, 0, 0), "id": 7, "key": 0},
+                    {"to_jimage": (0, 0, 0), "id": 14, "key": 0},
+                    {"to_jimage": (0, 0, 0), "id": 15, "key": 0},
+                ],
+                [{"to_jimage": (0, 0, 0), "id": 8, "key": 0}],
+                [{"to_jimage": (0, 0, 0), "id": 17, "key": 0}],
+                [
+                    {"to_jimage": (0, 0, 0), "id": 2, "key": 0},
+                    {"to_jimage": (0, 0, 0), "id": 6, "key": 0},
+                    {"to_jimage": (0, 0, 0), "id": 7, "key": 0},
+                ],
+                [
+                    {"to_jimage": (0, 0, 0), "id": 3, "key": 0},
+                    {"to_jimage": (0, 0, 0), "id": 14, "key": 0},
+                    {"to_jimage": (0, 0, 0), "id": 16, "key": 0},
+                ],
+                [
+                    {"to_jimage": (0, 0, 0), "id": 4, "key": 0},
+                    {"to_jimage": (0, 0, 0), "id": 9, "key": 0},
+                    {"to_jimage": (0, 0, 0), "id": 13, "key": 0},
+                ],
+                [{"to_jimage": (0, 0, 0), "id": 9, "key": 0}],
+                [{"to_jimage": (0, 0, 0), "id": 13, "key": 0}],
+                [
+                    {"to_jimage": (0, 0, 0), "id": 1, "key": 0},
+                    {"to_jimage": (0, 0, 0), "id": 2, "key": 0},
+                    {"to_jimage": (0, 0, 0), "id": 11, "key": 0},
+                ],
+            ],
+        },
+    }
+    mg = MoleculeGraph.from_dict(mg_dict)
+    mol = mg.molecule
+    linker = Linker(
+        mol,
+        mg,
+        center=np.array([-502.5498308, -406.76009494, -502.5498308]),
+        graph_branching_indices={8, 13, 17},
+        closest_branching_index_in_molecule={1, 37, 46},
+        binding_indices=[129, 137, 108, 141, 120, 152],
+        original_indices=[
+            96,
+            129,
+            23,
+            152,
+            60,
+            137,
+            105,
+            0,
+            1,
+            2,
+            108,
+            141,
+            69,
+            37,
+            14,
+            84,
+            120,
+            4,
+        ],
+        connecting_paths=[],
+    )
+    return linker
+
+
+@pytest.fixture()
+def get_node_object():
+    mof = MOF.from_cif(os.path.join(THIS_DIR, "test_files", "HKUST-1.cif"))
+    node = Node.from_mof_and_indices(
+        mof,
+        node_indices={32, 1, 132, 134, 72, 137, 74, 139, 108, 110, 113, 115, 27, 30},
+        branching_indices={1, 27, 30, 32},
+        binding_indices={108, 110, 113, 115, 132, 134, 137, 139},
+        connecting_paths={1, 27, 30, 32, 108, 110, 113, 115, 132, 134, 137, 139},
+    )
+
+    return node
